@@ -9,12 +9,12 @@ module Rack
     end
 
     def close
-      raise IOError, "closed stream" if @closed
+      return if @closed
+      @closed = true
       begin
         @body.close if @body.respond_to? :close
       ensure
         @block.call
-        @closed = true
       end
     end
 
